ADDRESS_BYTES_LENGTH = 33
ADDRESS_VERSION = b'\x42'
CONTRACT_VERSION = b'\xC0'

PRIVATE_KEY_VERSION = b'\xAA'

PUBLIC_KEY_UNCOMPRESSED = b'\x04'
PUBLIC_KEY_COMPRESSED_E = b'\x02'
PUBLIC_KEY_COMPRESSED_O = b'\x03'
